{{
  Subset Java Virtual Machine (Using adapted Javelin Linker and Jikes 0.47 compiler).
   
  From the javelin manual:
  Memory and Variable Types 
  This information may come in handy if your application is running short on memory space:  
  -  All types (including char and byte) use 2 bytes of memory.   
  -  Arrays require the amount of space to store their elements (that is, 2 times the number of elements) 
     plus 4 additional bytes.   
  -  Exception: byte and char arrays use one byte per element.   
  -  Local variables deduct from your stack and are reclaimed when they go out of scope.   
  -  Objects require space for their non-static fields, plus two bytes of overhead.   
 
  primitive  storage/value  supported 
  boolean     2bytes/1bit      yes    
  byte        2bytes/1byte     yes      byte[n] storage is 2 ints + n bytes
  char        2bytes/1byte     yes      char[n] storage is 2 ints + n bytes
  short       2bytes/2bytes    yes    
  int         2bytes/2bytes    yes    
  long        4bytes/4bytes    later  
  float       4bytes/4bytes    later  
  double      8bytes/8bytes    later  
                                        other arrays: 2 ints + n*storage bytes
                                        
  Registers
  The registers of the Subset Java Virtual Machine are similar to the registers in your computer. However,
  because the Virtual Machine is stack based, its registers are not used for passing or receiving arguments.
  In Java, registers hold the machine's state and are updated after each line of bytecode is executed to
  maintain that state. All these registers are 16 bits wide and are allocated immediately.
  The following four registers hold the state of the virtual machine:
 
  frame - reference frame, contains a pointer to the execution environment of the current method. 
  optop - operand top, contains a pointer to the top of the operand stack and is used to evaluate arithmetic expressions. 
  pc    - program counter, contains the address of the next bytecode to be executed. 
  vars  - variable register, contains a pointer to local variables. 

}}

OBJ
  debug: "MultiCogSerialDebug"
  jem  : "AiChip_JVM_JemFile_002"
  
CON
  MAX_NUM_VPS   = 6  'as defined in CPU.java
  nmPeriph0Bank = $80
  nmPeriph1Bank = nmPeriph0Bank + $10
  nmPeriph2Bank = nmPeriph0Bank + $20
  nmPeriph3Bank = nmPeriph0Bank + $30
  nmPeriph4Bank = nmPeriph0Bank + $40
  nmPeriph5Bank = nmPeriph0Bank + $50

DAT
  vpRambank     byte 0[MAX_NUM_VPS*16]   'registers for Virtual Peripherals (16 bytes per VP)

  frame         word 0   'reference frame 
  optop         word 0   'operand top 
  pc            word 0   'program counter 
  vars          word 0   'variable register 
  carry         word 0   'true if overflow/underflow after math operation

PUB do_native(nm_function): value
  'parameters for native functions need to be retrieved 
  case nm_function
     0: nm_core_CPU_writeRegister(0,0)
     1: return nm_core_CPU_readRegister(0)
     2: nm_core_CPU_writeObject(0,0)
     3: nm_core_CPU_delay(0)
     4: nm_core_CPU_message(0,0)
     5: nm_page_
     6: return nm_core_CPU_count(0,0,0)
     7: return nm_core_CPU_rcTime(0,0,0)
     8: return nm_core_CPU_pulseIn(0,0,0)
     9: nm_core_CPU_pulseOut(0,0)
    10: return nm_core_CPU_readPin(0)
    11: nm_core_CPU_writePin(0,0)
    12: return nm_core_CPU_shiftIn(0,0,0,0)
    13: nm_core_CPU_shiftOut(0,0,0,0,0)
    14: return nm_core_CPU_carry
    15: nm_page_
    16: nm_core_CPU_uninstall(0)
    17: nm_core_CPU_install(0)
    18: nm_core_CPU_nap(0)
    19: nm_core_CPU_setOutput(0)
    20: nm_page_
    21: nm_core_CPU_setInput(0)
    22: nm_page_
    23: return nm_core_CPU_readSRAM(0)
    24: nm_page_
    25: nm_core_CPU_writeSRAM(0,0)
    26: nm_page_
    27: return nm_core_EEPROM_eeRead(0)
    28: nm_page_
    29: nm_core_EEPROM_eeWrite(0,0)
    30: nm_page_
    31: nm_core_Uart_txInit(0)
    32: nm_core_Uart_rxInit(0)
    33: nm_core_Uart_rxRead(0)
    34: nm_core_Callback_returnTwo
    35: nm_core_Timer_init(0)
    36: nm_core_Timer_latch(0)
    37: nm_page_
    38: nm_core_PWM_install(0)
    39: nm_page_
    40: nm_core_PWM_updateInternal(0,0,0,0,0)
    41: nm_page_
    42: nm_core_DAC_install(0)
    43: nm_page_
    44: nm_core_ADC_install(0)
    45: nm_page_
    46: return nm_core_Terminal_getByte
    47: nm_page_


CON
  'port addresses used with readRegister and writeRegister
  sxRB = $06
  sxRC = $07

  PORTA = $00FF  'A constant representing the first I/O port on the Prop. This port contains pins 0-7.
  PORTB = $01FF  'A constant representing the second I/O port on the Prop. This port contains pins 8-15.


PRI nm_core_CPU_writeRegister(address,value)
  case address
    nmPeriph0Bank..(nmPeriph0Bank+MAX_NUM_VPS*16-1):
      vpRambank[address-nmPeriph0Bank] := value
    sxRB: outa[7..0] := value
    sxRC: outa[15..8] := value
    
  
PRI nm_core_CPU_readRegister(address):value
  case address
    nmPeriph0Bank..(nmPeriph0Bank+MAX_NUM_VPS*16-1):
      return vpRambank[address-nmPeriph0Bank]
    sxRB: return ina & $FF
    sxRC: return (ina >> 8) & $FF

  
PRI nm_core_CPU_writeObject(address,value)
  'Needs to write where ???


PRI nm_core_CPU_delay(period) | t
  '/**
  ' * Delays the program execution for a period of time. The delay method does not
  ' * return until the delay period has expired. During a delay any installed Virtual
  ' * Peripherals will continue to run.
  ' * <p>
  ' *
  ' * @param period the length of time to delay for, measured in
  ' * 95.48 microsecond units.
  ' */
  t := cnt
  repeat until (cnt - t) / (clkfreq * 95.48 / 1000000) > period

  
PRI nm_core_CPU_message(data,length)
  'write character array to debug output
  if length > 0
    repeat while length > 1
     debug.cprintf(string("%c"),byte[data],true)
      data++
      length--
    debug.cprintf(string("%c"),byte[data],false)


PRI nm_core_CPU_count(timeout,portPin,edge): value | pin,t
  '/**
  ' * Count the number of rising or falling edge transitions that occur during
  ' * a set period of time.
  ' * If the pin is an output then it will be changed to be an input and will remain as
  ' * an input when the counting is complete.
  ' * <p>
  ' * To ensure reliable detection the input signal must maintain a value for at least
  ' * 8.68us. This means that a square wave must have a period of at least 17.36us and
  ' * a frequency less than 57.6kHz.
  ' *
  ' * @param timeout the amount of time to count transitions over. Measured in
  ' *                8.68us units. The value of 32,767 allows a maximum measurement
  ' *                time of 284.4ms.
  ' * @param portPin     the I/O pin to measure transitions on.
  ' * @param edge    the type of edge to count. Pass true to count rising edges,
  ' *                or false to count falling edges.
  ' * @return the number of rising edges or falling edges that occured on the pin
  ' *         during the timeout period.
  ' */
  {{ adapted from object BS2_functions PUB COUNT (Pin, Duration) : Value }}
  {{ only does rising-edge pulses now  }}
  if portPin < 256
    pin := >|portPin
  else
    portPin &= $FF
    pin := 8 + >|portPin
  dira[pin]~                                          ' Set as input
  ctra := 0                                           ' Clear any value in ctrb
                                                           ' set up counter, pos edge trigger
  ctra := (%01010 << 26 ) | (%001 << 23) | (0 << 9) | (pin)  
  frqa := 1                                           ' 1 count/trigger
  phsa := 0                                           ' Clear phase - holds accumulated count
  t := cnt
  repeat until (cnt - t) / (clkfreq * 8.68 / 1000000) > timeout ' Allow to count for duration in 8.68us units 
  value := phsa                                       ' Return total count
  ctra := 0                                           ' clear counter mode


PRI nm_core_CPU_rcTime(timeout,portPin,pinState): value | pin, ClkStart,clkStop
  '/**
  ' * Measure the length of time it takes for a pin to match a desired value.
  ' * If the pin is an output then it will be changed to be an input and will
  ' * remain as an input when rcTime is complete. rcTime can be used to measure
  ' * the charge/discharge time of a RC (resistor-capacitor) circuit in order to
  ' * read an analogue value.
  ' *
  ' * @param timeout the maximum measurable time.
  ' * @param portPin the I/O pin to measure.
  ' * @param pinState the pin state to wait for.
  ' * @return the amount of time taken for the pin to change to the desired value,
  ' * measured in 8.68us units. A value of -1 indicates that the timeout expired
  ' * before the desired state was reached.
  ' */
  {{ adapted from object BS2_functions PUB RCTIME (Pin,State):Duration | ClkStart, ClkStop }}
  if portPin < 256
    pin := >|portPin
  else
    portPin &= $FF
    pin := 8 + >|portPin
  if pinState
    pinState := 1
  else
    pinState := 0
  dira[pin]~
  ClkStart := cnt                                         ' Save counter for start time
  waitpne(pinState << pin, |< pin, 0)                     ' Wait for opposite state to end
  clkStop := cnt                                          ' Save stop time
  value := (clkStop - ClkStart)/(clkfreq * 8.68 /1000000) ' calculate in 8.68us resolution


PRI nm_core_CPU_pulseIn(timeout,portPin,pinState): value | pin
  '/**
  ' * Measure the length of a single pulse. Waits for the value of the selected
  ' * pin to match the value selected by the state parameter and then times how
  ' * long it takes for the pin to return to its original state.
  ' * If the pin is an output then it will be changed to be an input and will
  ' * remain as an input when complete.
  ' * <p>
  ' * A pulse of less than 8.68us in duration may be missed.
  ' *
  ' * @param timeout  the maximum amount of time to spend in the pulseIn method.
  ' *                 The timeout covers the complete time of the process both
  ' *                 the wait for the start of the pulse and the time of the pulse.
  ' * @param portPin      the I/O pin to measure the pulse on.
  ' * @param pinState    whether to measure a high pulse (true) or a low pulse (false).
  ' * @return the length of the pulse in 8.68us units. A value of 0 indicates
  ' *         that no start edge was detected. A value of -1 indicates that no
  ' *         stop edge was detected.
  ' */
  {{ adapted from object BS2_functions PUB PULSIN_Clk(Pin, State) : Duration }}
  {{ need to implement timeout }}
  if portPin < 256
    pin := >|portPin
  else
    portPin &= $FF
    pin := 8 + >|portPin
  dira[pin]~
  ctra := 0
  if pinState
    pinState := 1
  else
    pinState := 0
  if pinState == 1
    ctra := (%11010 << 26 ) | (%001 << 23) | (0 << 9) | (pin) ' set up counter, A level count
  else
    ctra := (%10101 << 26 ) | (%001 << 23) | (0 << 9) | (pin) ' set up counter, !A level count
  frqa := 1
  waitpne(pinState << pin, |< Pin, 0)                         ' Wait for opposite state ready
  phsa:=0                                                  ' Clear count
  waitpeq(pinState << pin, |< Pin, 0)                         ' wait for pulse
  waitpne(pinState << pin, |< Pin, 0)                         ' Wait for pulse to end
  value := phsa * 1000000 /clkfreq / 8.68                  ' Return duration as 8.68us units
  ctra :=0                                                 ' stop counter


CON
  cntMin     = 400      ' Minimum waitcnt value to prevent lock-up


PRI nm_core_CPU_pulseOut(length,portPin) | ClkCycles,pin
  '/**
  ' * Generate a pulse of a specific duration. If the pin is an input then it will
  ' * be changed to be an output and remain as an output when complete. To
  ' * ensure that a positive going pulse or negative going pulse is generated it
  ' * is a good idea to write a value to the pin first to put it into the correct
  ' * state. pulseOut inverts the value of the selected pin, waits for the
  ' * specified length of time, and then returns the pin to its original value.
  ' * <p>
  ' * The time is measured in 8.68us units with a maximum value of 32767 giving a
  ' * maximum pulse time of 284.4ms. A time of 0 will not cause a pulse.
  ' * pulseOut can be used to generate the control pulses for a servo motor.
  ' *
  ' * @param length the length of the pulse to generate, measured in 8.68us units.
  ' * @param portPin    the I/O to generate the pulse on.
  ' */
  {{ adapted from object BS2_functions PUB PULSOUT(Pin,Duration)  | clkcycles }}
  if portPin < 256
    pin := >|portPin
  else
    portPin &= $FF
    pin := 8 + >|portPin
  ClkCycles := (length * clkfreq / 1000000 * 8.68 - 1250) #> cntMin        ' duration * clk cycles for 8.68us
                                                           ' - inst. time, min cntMin
  dira[pin]~~                                              ' Set to output                                         
  !outa[pin]                                               ' set to opposite state
  waitcnt(clkcycles + cnt)                                 ' wait until clk gets there 
  !outa[pin]                                               ' return to orig. state


CON
  'pins
  pin0  = 1<<0
  pin1  = 1<<1
  pin2  = 1<<2
  pin3  = 1<<3
  pin4  = 1<<4
  pin5  = 1<<5
  pin6  = 1<<6
  pin7  = 1<<7
  pin8  = (1<<0)+256
  pin9  = (1<<1)+256
  pin10 = (1<<2)+256
  pin11 = (1<<3)+256
  pin12 = (1<<4)+256
  pin13 = (1<<5)+256
  pin14 = (1<<6)+256
  pin15 = (1<<7)+256
  
  
PRI nm_core_CPU_readPin(portPin): YesNo
  '/**
  ' * Reads the logic value on a pin. If the pin is an output then it will be
  ' * changed to be an input and will remain as an input when complete.
  ' *
  ' * @param portPin the I/O pin to read.
  ' * @return the state of the pin where true indicates high and false indicates low.
  ' */
  if portPin < 256
    dira[7..0] &= (255-portPin)
    return ina[>|portPin]
  else
    portPin -=256
    dira[15..8] &= (255-portPin)
    return ina[8 + >|portPin]
    

PRI nm_core_CPU_writePin(portPin,value)
  '/**
  ' * Set a pin to a logic state. Makes the selected pin high or low as selected
  ' * by the state parameter. If the pin is an input then it will be changed to
  ' * be an output and remain as an output when complete.
  ' *
  ' * @param portPin the I/O pin to write.
  ' * @param value the state to set the pin to (true=high, false=low).
  ' */
  if portPin < 256
    dira[7..0] |= portPin
    outa[>|portPin] := value
  else
    portPin -=256
    dira[15..8] |= portPin
    outa[8 + >|portPin] := value


CON
  'shiftin modes
  PRE_CLOCK_MSB  = 0  'Specify pre-clock sampling and MSB-first transmission. 
  POST_CLOCK_MSB = 1  'Specify post-clock sampling and MSB-first transmission.
  PRE_CLOCK_LSB  = 2  'Specify pre-clock sampling and LSB-first transmission.
  POST_CLOCK_LSB = 3  'Specify post-clock sampling and LSB-first transmission.
  OnClock        = 4

PRI nm_core_CPU_shiftIn(dataPortPin,clockPortPin,bitCount,mode): value | Dpin,Cpin,InBit
  '/**
  ' * Read data from a synchronous serial device.
  ' * The clock pin outputs a clock signal and the data is clocked in form the data pin.
  ' * If the data pin is an output then it will be changed to be an input and remain as
  ' * an input when complete. If the clock pin is an input then it will be changed to be
  ' * an output and remain as an output when complete.
  ' * <p> The mode parameter selects pre/post clock sampling. Note that some devices
  ' * require pre-clock sampling. When pre-clock sampling is used the data pin is
  ' * sampled to obtain the first bit prior to any change of the clock pin. Clearly
  ' * the device must have had a clock pulse to output the first bit and this is
  ' * normally supplied by the previous call to shiftIn or shiftOut. Most
  ' * synchronous serial devices require a write (shiftOut) prior to reading and so the
  ' * clock for the first bit will be supplied by the shiftOut call. If the device must be
  ' * read without first being written to then the first read (shiftIn) should request an
  ' * extra bit to provide the initial clock pulse.
  ' * <p>
  ' * Note that the bit count can be set to higher than 16 (up to 256) if desired. When
  ' * a number greater than 16 is used the last 16 bits will be returned.
  ' *
  ' * @param dataPortPin the I/O pin to read data on.
  ' * @param clockPortPin the I/O pin on which to provide the clock to the external device.
  ' * @param bitCount the number of bits to count in (from 1 to 16). bitCount must not be 0.
  ' * @param mode the shifting and clocking mode to employ. This determines whether
  ' * bits are shifted into the least-significant-bit (LSb) or most-significant-bit (MSb)
  ' * first. The clocking mode determines whether the data is sampled before or after
  ' * the first clock transition. See the description for the possible values.
  ' * @return the bits shifted in on the data pin. The result contains bitCount bits and is
  ' * justified according to the mode setting. If MSb first is selected then the result
  ' * will be the data read in on the port. If LSb first is selected then the data is
  ' * returned MSb justified and must be shifted right (16-bitCount) bits to
  ' * obtain the correct value.
  ' */
  {{ adapted from object BS2_functions PUB SHIFTIN (Dpin, Cpin, Mode, Bits) : Value | InBit }}
{{
   Shift data in, master clock, for mode use BS2#MSBPRE, #MSBPOST, #LSBPRE, #LSBPOST
   Clock rate is ~16Kbps.  Use at 80MHz only is recommended.
     X := BS2.SHIFTIN(5,6,BS2#MSBPOST,8)
}}
  if dataPortPin < 256
    Dpin := >|dataPortPin
  else
    dataPortPin &= $FF
    Dpin := 8 + >|dataPortPin
  if clockPortPin < 256
    Cpin := >|clockPortPin
  else
    clockPortPin &= $FF
    Cpin := 8 + >|clockPortPin
    dira[Dpin]~                                            ' Set data pin to input
    outa[Cpin]:=0                                          ' Set clock low 
    dira[Cpin]~~                                           ' Set clock pin to output 
                                                
  If Mode == PRE_CLOCK_MSB                               ' Mode - MSB, before clock
       Value:=0
       REPEAT Bitcount                                         ' for number of bits
          InBit:= ina[Dpin]                                ' get bit value
          Value := (Value << 1) + InBit                    ' Add to  value shifted by position
          !outa[Cpin]                                      ' cycle clock
          !outa[Cpin]
          waitcnt(1000 + cnt)                              ' time delay

  elseif Mode == POST_CLOCK_MSB                            ' Mode - MSB, after clock              
       Value:=0                                                          
       REPEAT Bitcount                                         ' for number of bits                    
          !outa[Cpin]                                      ' cycle clock                         
          !outa[Cpin]                                         
          InBit:= ina[Dpin]                                ' get bit value                          
          Value := (Value << 1) + InBit                    ' Add to  value shifted by position                                         
          waitcnt(1000 + cnt)                              ' time delay                            
                                                                 
  elseif Mode == POST_CLOCK_LSB                            ' Mode - LSB, after clock                    
       Value:=0                                                                                         
       REPEAT Bitcount                                         ' for number of bits                         
          !outa[Cpin]                                      ' cycle clock                          
          !outa[Cpin]                                                                             
          InBit:= ina[Dpin]                                ' get bit value                        
          Value := (InBit << (bitcount-1)) + (Value >> 1)      ' Add to  value shifted by position    
          waitcnt(1000 + cnt)                              ' time delay                           

  elseif Mode == PRE_CLOCK_LSB                             ' Mode - LSB, before clock             
       Value:=0                                                                                   
       REPEAT Bitcount                                         ' for number of bits                   
          InBit:= ina[Dpin]                                ' get bit value                        
          Value := (Value >> 1) + (InBit << (bitcount-1))      ' Add to  value shifted by position    
          !outa[Cpin]                                      ' cycle clock                          
          !outa[Cpin]                                                                             
          waitcnt(1000 + cnt)                              ' time delay

  elseif Mode == OnClock  {{ is this required ?? }}                                          
       Value:=0
       REPEAT Bitcount                                         ' for number of bits
                                        
          !outa[Cpin]                                      ' cycle clock
          waitcnt(500 + cnt)                               ' get bit value
          InBit:= ina[Dpin]                               ' time delay
          Value := (Value << 1) + InBit                    ' Add to  value shifted by position
          !outa[Cpin]
          waitcnt(500 + cnt)                           
     



CON
  'shiftout modes
  SHIFT_MSB = 0  'Specify MSB-first transmission.
  SHIFT_LSB = 2  'Specify LSB-first transmission. 


PRI nm_core_CPU_shiftOut(dataPortPin,clockPortPin,bitCount,mode,data) | Dpin,Cpin
  '/**
  ' * Write data to a synchronous serial device.
  ' * The clock pin outputs a clock signal and the data is clocked out the data pin. If
  ' * the data pin is an input then it will be changed to be an output and remain as an
  ' * output when complete. Likewise if the clock pin is an input then it will be
  ' * changed to be an output and remain as an output when complete.
  ' * <p>
  ' * Both the clock and data pins should be initialized before calling shiftOut. shiftOut
  ' * works by inverting the pins so you can generate either idle high or idle low signals.
  ' * <p>
  ' * If LSb first is selected then the data can be passed directly. If MSb first is
  ' * selected then if less than 16 bits are to be shift out, data should be shifted left
  ' * to MSb justify it.
  ' * @param dataPortPin the I/O pin to write data on.
  ' * @param clockPortPin the I/O pin on which to provide the clock to the external device.
  ' * @param bitCount the number of bits to count out (from 1 to 16). bitCount must not be 0.
  ' * @param mode the shifting mode to employ. This determines whether
  ' * bits are shifted into the least-significant-bit (LSb) or most-significant-bit (MSb)
  ' * first. Valid values are CPU.SHIFT_MSB and CPU.SHIFT_LSB.
  ' */
  {{ adapted from object BS2_functions PUB SHIFTOUT (Dpin, Cpin, Value, Mode, Bits)| bitNum  }}
{{
   Shift data out, master clock, for mode use ObjName#LSBFIRST, #MSBFIRST
   Clock rate is ~16Kbps.  Use at 80MHz only is recommended.
     BS2.SHIFTOUT(5,6,"B",BS2#LSBFIRST,8)
}}
  if dataPortPin < 256
    Dpin := >|dataPortPin
  else
    dataPortPin &= $FF
    Dpin := 8 + >|dataPortPin
  if clockPortPin < 256
    Cpin := >|clockPortPin
  else
  clockPortPin &= $FF
  Cpin := 8 + >|clockPortPin
  outa[Dpin]:=0                                          ' Data pin = 0
  dira[Dpin]~~                                           ' Set data as output
  outa[Cpin]:=0
  dira[Cpin]~~

  If Mode == SHIFT_LSB                                    ' Send LSB first    
       REPEAT Bitcount
          outa[Dpin] := data                              ' Set output
          data := data >> 1                              ' Shift value right
          !outa[Cpin]                                      ' cycle clock
          !outa[Cpin]
          waitcnt(1000 + cnt)                              ' delay

  elseIf Mode == SHIFT_MSB                                ' Send MSB first               
       REPEAT Bitcount                                                                
          outa[Dpin] := data >> (bitcount-1)                  ' Set output           
          data := data << 1                              ' Shift value right    
          !outa[Cpin]                                      ' cycle clock          
          !outa[Cpin]                                                             
          waitcnt(1000 + cnt)                              ' delay                
    outa[Dpin]~                                            ' Set data to low
     

  
PRI nm_core_CPU_carry: YesNo
  return carry

  
PRI nm_core_CPU_uninstall(bank)
  '/**
  ' * This function is called by CPU.removeVP. 
  ' * Remove a virtual peripheral from its slot. The VP will stop executing
  ' * immediately and its slot will be available to install a different VP.
  ' *
  ' * @param bank the rambank allocated to the virtual peripheral that must stop.
  ' */


PRI nm_core_CPU_install(routine)
  'Appears not to be used

  
PRI nm_core_CPU_nap(period)
  'If the Prop cannot support nap, then just do a delay
  '/**
  ' * Put the Prop into sleep mode for a period of time. The accuracy
  ' * of the sleep timer is very poor. See the manual for more information.
  ' *
  ' * @param period specifies the period for which the Prop should sleep. The
  ' * following values are valid:
  ' *
  ' * 0 - 16ms
  ' * 1 - 32ms
  ' * 2 - 64ms
  ' * 3 - 128ms
  ' * 4 - 256ms
  ' * 5 - 512ms
  ' * 6 - 1024ms
  ' * 7 - 2048ms
  ' */


PRI nm_core_CPU_setOutput(portPin)
  '/**
  ' * Make a pin an output.
  ' * The specified pin will be converted to an output. More than one pin can
  ' * be specified using the | operator, as long as all pins are on the same
  ' * port.
  ' *
  ' * @param portPin the pin(s) to make into an output.
  ' */
  if portPin < 256
    dira[7..0] |= portPin
  else
    portPin &= $FF
    dira[15..8] |= portPin
    

PRI nm_core_CPU_setInput(portPin)
  '/**
  ' * Make a pin an input.
  ' * The specified pin will be converted to an input. More than one pin can
  ' * be specified using the | operator, as long as all pins are on the same
  ' * port.
  ' *
  ' * @param portPin the pin(s) to make into an input.
  ' */
  if portPin < 256
    dira[7..0] &= (255 ^ portPin)
  else
    portPin &= $FF
    dira[15..8] &= (255 ^ portPin)


PRI nm_core_CPU_readSRAM(address): value
  '/**
  ' * Reads a byte from SRAM at the specified address.
  ' *
  ' * @param address the address to read the byte from.
  ' * @return the byte read from the SRAM.
  ' */
  return byte[address]
  

PRI nm_core_CPU_writeSRAM(address,value)
  '/**
  ' * Writes a byte into the SRAM at the specified address.
  ' *
  ' * @param address the address to write the byte to.
  ' * @param value value to write.
  ' */
  byte[address] := value
  

CON
  propSDA = 29
  propSCL = 28


PRI nm_core_EEPROM_eeRead(address): value | ack,temp
  '/**
  ' * Reads a byte from the EEPROM at the specified address.
  ' *
  ' * @param address the address to read the byte from.
  ' * @return the byte read from the EEPROM.
  ' */
  {{ adapted from object BS2_functions Pub Read(address, size) }}
  value~
  dira[propSCL]~~
  outa[propSCL]~~
  dira[propSDA]~
  outa[propSDA]~
  dira[propSDA]~~
  nm_core_CPU_shiftout(propSDA, propSCL, %10100000, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_LSB, 1)
  nm_core_CPU_shiftout(propSDA, propSCL, address >> 8, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_LSB, 1)
  nm_core_CPU_shiftout(propSDA, propSCL, address, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_LSB, 1)
  dira[propSCL]~~
  outa[propSCL]~~
  dira[propSDA]~
  outa[propSDA]~
  dira[propSDA]~~
  nm_core_CPU_shiftout(propSDA, propSCL, %10100001, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_LSB, 1)
  dira[propSDA]~
  value := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_MSB, 8)
  ack := nm_core_CPU_shiftin(propSDA, propSCL, PRE_CLOCK_LSB, 1)
  outa[propSDA]~
  dira[propSDA]~~
  outa[propSCL]~~
  dira[propSDA]~
        

PRI nm_core_EEPROM_eeWrite(address,value) | ack 
  '/**
  ' * Writes a byte into the EEPROM at the specified address.
  ' *
  ' * @param address the address to write the byte to.
  ' * @param value value to write.
  ' */
  {{ adapted from object BS2_functions Pub Write(address, value, size) }}
  outa[propSCL]~~
  dira[propSCL]~~
  dira[propSDA]~
  outa[propSDA]~
  dira[propSDA]~~
  nm_core_CPU_shiftout(propSDA, propSCL, %10100000, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL,OnClock,1)
  nm_core_CPU_shiftout(propSDA, propSCL, address >> 8, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL,OnClock, 1)
  nm_core_CPU_shiftout(propSDA, propSCL, address, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL,OnClock,1)
  nm_core_CPU_shiftout(propSDA, propSCL, value, SHIFT_MSB, 8)
  dira[propSDA]~
  ack := nm_core_CPU_shiftin(propSDA, propSCL,OnClock, 1)
  outa[propSDA]~
  dira[propSDA]~~
  outa[propSCL]~~
  dira[propSDA]~
  waitcnt(clkfreq/200 + cnt)
           


CON
  'Uart registers
  nmUartDataH    = $02
  nmUartDataL    = $03
  nmUartInvert   = $05 '// 0 for normal, 1 for invert
  nmUartBaud     = $06 '// 1000000/(bps * 4.34)
  nmUartStopBits = $07 '// Tx only: 00000001 for 1 stop bit, 00000011 for 2 etc
  nmUartPort     = $0C '// 0 for Port0, 1 for Port1
  nmUartPin      = $0D '// bit mask (1 of 8 set)
  nmUartHSPort   = $0E '// 0 for Port0, 1 for Port1
  nmUartHSPin    = $0F '// bitmask (1 of 8 set)
  nmUartBufferH  = $0A
  nmUartBufferL  = $0B
  nmUartHead     = $08 '//Index of the next free byte in the buffer. 
  nmUartTail     = $09 '//Index of the oldest byte in the buffer.
                       '//If tailPointer == headPointer the buffer is empty.

  'Uart baud constants
  speed600   = 192  'The speed value for 600 baud.
  speed1200  = 96   'The speed value for 1200 baud.
  speed2400  = 48   'The speed value for 2400 baud.
  speed4800  = 24   'The speed value for 4800 baud.
  speed7200  = 16   'The speed value for 7200 baud.
  speed9600  = 12   'The speed value for 9600 baud.
  speed14400 = 8    'The speed value for 14400 baud.
  speed19200 = 6    'The speed value for 19200 baud.
  speed28800 = 4    'The speed value for 28800 baud.
  speed38400 = 3    'The speed value for 38400 baud.
  speed57600 = 2    'The speed value for 57600 baud.


PRI nm_core_Uart_txInit(bank)
  '/**
  ' * Initialize a Uart transmitter as background task. Its parameters are in vpRambank.
  ' * Called by Uart.start()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  

PRI nm_core_Uart_rxInit(bank)
  '/**
  ' * Initialize a Uart receiver as background task. Its parameters are in vpRambank.
  ' * Called by Uart.start()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]


PRI nm_core_Uart_rxRead(bank)
  '/**
  ' * Inform Uart receiver a byte has been grabbed from the buffer, so the reciever can do the handshaking.
  ' * Called by Uart.readByte()
  ' *
  ' * @param bank The allocated area for the Uart registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]


PRI nm_core_Callback_returnTwo
  'This appears not to be used

  
CON
  'Timer registers - Note that the Prop already has a running 32bits timer
  nmPeriphISRVec = $00
  nmLatchTimer1  = $06
  nmLatchTimer2  = $07
  nmLatchTimer3  = $08
  nmLatchTimer4  = $09
  nmTimer1       = $02
  nmTimer2       = $03
  nmTimer3       = $04
  nmTimer4       = $05


PRI nm_core_Timer_init(bank)
  '/**
  ' * Initialize the 32bit counter. The 32bit nmTimer register should increment every 8.68 usec.
  ' * Called by Timer.start()
  ' *
  ' * @param bank The allocated area for the Timer registers.
  ' */


PRI nm_core_Timer_latch(bank)
  '/**
  ' * Latch the 32bit counter in the latch registers.
  ' * Called by Timer.tickLo()
  ' *
  ' * @param bank The allocated area for the Timer registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  'This probably needs a lock
  vpRambank[bank + nmLatchTimer1] := vpRambank[bank + nmTimer1]
  vpRambank[bank + nmLatchTimer2] := vpRambank[bank + nmTimer2]
  vpRambank[bank + nmLatchTimer3] := vpRambank[bank + nmTimer3]
  vpRambank[bank + nmLatchTimer4] := vpRambank[bank + nmTimer4]
  

CON
  'PWM registers
  nmPWMPort = $0E '// 0 for Port0, 1 for Port1
  nmPWMPin  = $0F '// bitmask (1 of 8 set)
  'not defined in java sources but we can use unused registers
  nmPWMhighTime = $0A
  nmPWMlowTime  = $0B
  nmPWMhighZero = $0C
  nmPWMlowZero  = $0D


PRI nm_core_PWM_install(bank) | pin,low,high,duty,scale
  '/**
  ' * Initialize a PWM as background task. Its parameters are in vpRambank.
  ' * Called by PWM.start()
  ' *
  ' * @param bank The allocated area for the PWM registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  {{ adapted from object BS2_functions Pub PWM_Set(Pin, Duty, Resolution) | Scale }}
  {{ Produces constant PWM on the specified pin }}
  {{ Needs to calculate scale }}
  if vpRambank[bank + nmPWMPort] == 0
    pin := >|vpRambank[bank + nmPWMPin]
  else
    pin := 8 + >|vpRambank[bank + nmPWMPin]
  low := vpRambank[bank + nmPWMlowTime]
  high := vpRambank[bank + nmPWMhighTime]
  duty := (high*100) / (low + high)
  if duty == 0                                         ' freq = 0 turns off square wave
     ctra := 0                                          ' Set CTRA/B to 0
     dira[pin]~                                         ' Make pin input
  else
      'scale := 2_147_483_647 / (1<< (Resolution-1))  ' Calculate scale
      ctra[30..26] := %00110      ' Set ctra to DUTY mode
      ctra[5..0] := pin           ' Set ctra's APIN
      frqa := duty * scale        ' Set frqa register
      dira[pin]~~                 ' set direction


PRI nm_core_PWM_updateInternal(bank,highTime,lowTime,highZero,lowZero)
  '/**
  ' * Update the PWM duty cycle. Its parameters are in vpRambank. 
  ' * Called by PWM.update()
  ' *
  ' * @param bank The allocated area for the PWM registers.
  ' * @param highTime the number of 8.68us the waveform should be high.
  ' * @param lowTime the number of 8.68us the waveform should be low.
  ' * @param highZero Boolean value (always true) that tells wether highTime != 0.
  ' * @param lowZero Boolean value (always true) that tells wether lowTime != 0.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  vpRambank[bank + nmPWMlowTime] := lowTime
  vpRambank[bank + nmPWMhighTime] := highTime
  vpRambank[bank + nmPWMlowZero] := lowZero
  vpRambank[bank + nmPWMhighZero] := highZero
  {{ Need to update ctra and frqa }}
  

CON
  'DAC registers
  nmDacValue = $0B '// Current ADC value.
  nmDacPort  = $0E '// 0 for Port0, 1 for Port1
  nmDacPin   = $0F '// bitmask (1 of 8 set)


PRI nm_core_DAC_install(bank)
  '/**
  ' * Initialize a DAC as background task. Its parameters are in vpRambank.
  ' * Called by DAC.start()
  ' *
  ' * @param bank The allocated area for the DAC registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  {{ Need some 8bit DAC routine }}


CON
  'ADC registers
  nmAdcValue = $0B '// Current ADC value.
  nmInPort   = $0C '// 0 for Port0, 1 for Port1
  nmInPin    = $0D '// bitmask (1 of 8 set)
  nmOutPort  = $0E '// 0 for Port0, 1 for Port1
  nmOutPin   = $0F '// bitmask (1 of 8 set)


PRI nm_core_ADC_install(bank)
  '/**
  ' * Initialize an ADC as background task. Its parameters are in vpRambank.
  ' * Called by ADC.start()
  ' *
  ' * @param bank The allocated area for the ADC registers.
  ' */
  bank -= nmPeriph0Bank
  'Access the proper register using vpRambank[bank+register]
  {{ Need some 8bit ADC routine }}

  
PRI nm_core_Terminal_getByte
  '/**
  ' * Request a byte from the debug terminal.
  ' * Called by Terminal.internalGetChar()
  ' *
  ' * @return The byte sent by the debug terminal.
  ' */
  return debug.rxcheck
  

PRI nm_page_
  'Since the Prop has no pages, simply ignore ???
  
